import 'package:flutter/material.dart';

class JamesBarPositioned extends StatefulWidget {
  final double? top;
  final double? left;
  final double? right;
  final double? bottom;
  final Axis axis;
  final List<Widget> items;
  final void Function(int) cb;
  final Color color;
  final Widget closeIcon;
  const JamesBarPositioned({
    Key? key,
    this.axis = Axis.horizontal,
    required this.items,
    required this.cb,
    required this.color,
    required this.closeIcon,
    required this.top,
    required this.left,
    required this.right,
    required this.bottom
  }) : super(key: key);

  @override
  State<JamesBarPositioned> createState() => _JamesBarPositionedState();
}

class _JamesBarPositionedState extends State<JamesBarPositioned> with SingleTickerProviderStateMixin{

  AnimationController? _act;
  Animation<double>? _anim;

  bool check = false;

  @override
  void initState() {
    _act = AnimationController(vsync: this, duration: const Duration(milliseconds: 300))
      ..addListener(() {
        if(!mounted) return;
        setState((){});
      });
    super.initState();
  }

  @override
  void dispose() {
    _act?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) => Positioned(
    top: widget.top,
    left: widget.left,
    right: widget.right,
    bottom: widget.bottom,
    child: widget.axis == Axis.vertical ? _vertical(context) : _horizontal(context)
  );

  Widget _vertical(BuildContext context){
    _anim = Tween<double>(begin:0, end: (-1*MediaQuery.of(context).size.height)+(kToolbarHeight)+MediaQuery.of(context).padding.top+200.0).animate(_act!);
    return Transform.translate(
      offset: Offset(0,_anim?.value ?? 0),
      child: Card(
        shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(20.0)),
        elevation: 4.0,
        color: widget.color,
        margin: const EdgeInsets.symmetric(vertical: 60.0, horizontal: 10.0),
        child: Container(
            width: 80.0,
            padding: const EdgeInsets.symmetric(vertical: 30.0),
            height: MediaQuery.of(context).size.height-kToolbarHeight-MediaQuery.of(context).padding.top,
            child: Column(
              children: [
                Expanded(
                  child: Column(
                    mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                    children: [..._bar()],
                  ),
                ),
                Container(child: _trigger()),
              ],
            )
        ),
      ),
    );
  }

  Widget _trigger() => IconButton(
    icon: widget.closeIcon as Icon,
    onPressed: (){
      if(!check) {_act?.forward();}
      else {_act?.reverse();}
      check = !check;
    },
  );

  Widget _horizontal(BuildContext context){
    _anim = Tween<double>(begin: 0, end: -1*MediaQuery.of(context).size.width+140.0).animate(_act!);
    return Transform.translate(
      offset: Offset(_anim?.value??0,0),
      child: Card(
        margin: const EdgeInsets.symmetric(vertical: 10.0, horizontal: 60.0),
        shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(20.0)),
        elevation: 4.0,
        color: widget.color,
        child: Container(
          width: MediaQuery.of(context).size.width,
          padding: const EdgeInsets.symmetric(horizontal: 30.0),
          height: 80.0,
          child: Row(
            children: [
              Expanded(
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                  children: [..._bar()],
                ),
              ),
              Container(child: _trigger()),
            ],
          )
        ),
      ),
    );
  }

  List<Widget> _bar() => widget.items.map<Widget>((Widget item) => InkWell(
    onTap: () => widget.cb(widget.items.indexOf(item)),
    child: item,
  )).toList();
}